class Light {
  /*
  Our mobile light source.
   */
  // predefined colors that can be used if more than one light is
  // created.  Only allows for 3 lights max, unless you add more colors.
  color colors[] = {
    color(64, 0, 255), color(0, 64, 255), color(64, 64, 255)
  };
  int[] col;
  // number of sphere's to draw in the trail:
  int lightTrail = 64;
  ArrayList trails = new ArrayList(); 
  // Size of this light:
  float lsize = maxSize/32;

  // Help to give our light an initial 'push' before it starts orbiting:
  float dirForce = 20;
  PVector pos = new PVector(random(-maxSize/2, maxSize/2), random(-maxSize/2, maxSize/2), random(-maxSize/2, maxSize/2));
  PVector dir = new PVector(random(-dirForce, dirForce), random(-dirForce, dirForce), random(-dirForce, dirForce));

  Light(int colorIndex) {
    color c = colors[colorIndex];
    col = new int[] {
      int(red(c)), int(green(c)), int(blue(c))
    };
    for (int i=0; i<lightTrail-1; i++) {
      trails.add(pos.get());
    }
  }

  void compute() {
    update();
    render();
  }

  void update() {
    PVector vecToCenter = PVector.sub(gCenter, pos);
    vecToCenter.normalize();
    vecToCenter.mult(gGravity);
    dir.add(vecToCenter);
    pos.add(dir);

    // fill our buffer of light trails:
    trails.add(pos.get());
    trails.remove(0);
  }

  void render() {

    // Draw the trail:
    noFill();
    stroke(col[0], col[1], col[2]);
    beginShape();
    for (int i=0; i<lightTrail-1; i++) {
      // Scale the tail based on its distane from the light
      float sVal = float(i)/float(lightTrail);
      
      PVector trailPos = (PVector) trails.get(i);
      // Stroke weights are a fixed size on screen, regardless of
      // zoom.  So we need to fake the zoom effect.
      float[] camPos = cam.getPosition();
      float distToCam = dist(trailPos.x, trailPos.y, trailPos.z, camPos[0], camPos[1], camPos[2]);
      float mult = 1/(distToCam / (lsize*2)) * gStrokeMult;
      strokeWeight(sVal*lsize*mult);
      curveVertex(trailPos.x, trailPos.y, trailPos.z);
    }
    curveVertex(pos.x, pos.y, pos.z);
    endShape();
    noStroke();

    // Draw the main light:
    sphereDetail(16);
    pushMatrix();
    translate(pos.x, pos.y, pos.z);
    fill(col[0], col[1], col[2]);
    
    sphere(lsize);
    pointLight(col[0], col[1], col[2], 0, 0, 0);
    
    //pointLight(col[0], col[1], col[2], 0, 0, 0);
    popMatrix();
  }
}

